@@ -1,6 +1,6 @@
 /*
 	conffile.h	Configuration File Functions Header
-	Copyright (c) 2000 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2002 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,10 @@
 #include "config.h"
 #include "confobj.h"
 
+bool	IsProfileValid(const string &s, bool allowGlobal);
+void	SetCurrentProfile(const string &s);
+string	GetCurrentProfile();
+
 void	NeedSecureUserConfig();
 void	LoadAllConfig(int quiet);
 void	PrintConfig();
