@@ -1,6 +1,6 @@
 /*
 	dirobj.cc	Directory Object Class
-	Copyright (c) 1996,1997-9,2000 Kriang Lerdsuwanakij
+	Copyright (c) 1996,1997-9,2000,2002,2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -79,7 +79,6 @@ DirectoryEntry::DirectoryEntry(gzFile file, char id) :
 			k_gzgetT(file, type);
 			k_gzgetT(file, modTime);
 			k_gzgetT(file, changeTime);
-			k_gzgetT(file, bookmarked);
 			break;
 			
 		case 'D':
@@ -92,7 +91,6 @@ DirectoryEntry::DirectoryEntry(gzFile file, char id) :
 			k_gzgetT(file, type);
 			k_gzgetT(file, modTime);
 			k_gzgetT(file, changeTime);
-			k_gzgetT(file, bookmarked);
 
 				// For dir. linkType == type and link is ""
 			link = "";
@@ -195,7 +193,6 @@ void	DirectoryEntry::gzWriteInfo(gzFile file)
 	k_gzputT(file, type);
 	k_gzputT(file, modTime);
 	k_gzputT(file, changeTime);
-	k_gzputT(file, bookmarked);
 }
 
 void	DirectoryEntry::gzWrite(gzFile file)
