@@ -1,6 +1,6 @@
 /*
 	gentree.cc	Generate Directory Tree For Display
-	Copyright (c) 2000, 2002 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2002, 2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -400,9 +400,15 @@ struct MatchInfoIsBookmarked {
 };
 
 struct MatchInfoIsNotBookmarkedNorCurDir {
-	bool operator()(MatchInfo &m) const {
-		return !m.dir->bookmarked && !m.is_curdir;
-	}
+	private:
+		string curdir;
+	public:
+		MatchInfoIsNotBookmarkedNorCurDir(const string &s) : curdir(s) {
+		}
+		bool operator()(MatchInfo &m) const {
+			return !m.dir->bookmarked && !m.is_curdir
+				&& m.name != curdir;
+		}
 };
 
 /*************************************************************************
@@ -663,7 +669,7 @@ FindDirInfo *RecursiveFindDir(sptr<DirectoryEntry> &d, const string &current_dir
 				if (str != new_dir)
 					continue;	// Not matched
 							// Skip this
-				return new FindDirInfo((*iter)(), str);	// Matched
+				return new FindDirInfo((*iter), str);	// Matched
 			}
 
 				// Now new_dir.size() < str.size()
@@ -726,7 +732,7 @@ FindDirInfo *RecursiveFindDir(sptr<DirectoryEntry> &d, const string &current_dir
 							// Ignore the `/'
 			ExtendTree(d, str.substr(current_dir.size()+1));
 		}
-		return new FindDirInfo(d(), current_dir);
+		return new FindDirInfo(d, current_dir);
 	}
 	else
 		return 0;
@@ -759,7 +765,7 @@ FindDirInfo *FindDir(const string &str, bool honor_cfg, bool change_top,
 				if (str != new_dir)
 					continue;	// Not matched
 							// Skip this
-				return new FindDirInfo((*iter)(), str);	// Matched
+				return new FindDirInfo(*iter, str);	// Matched
 			}
 
 				// Now new_dir.size() < str.size()
@@ -798,7 +804,7 @@ FindDirInfo *FindDir(const string &str, bool honor_cfg, bool change_top,
 		if (kcdConfig.cfgSortTree)
 			dirTree.sort();
 
-		return new FindDirInfo(d(), str);
+		return new FindDirInfo(d, str);
 	}
 	else
 		return 0;
@@ -813,6 +819,8 @@ FindDirInfo *FindDir(const string &str, bool honor_cfg, bool change)
 	Change directory
 *************************************************************************/
 
+void	GetBookmarks(list<MatchInfo> &match_list, const string &linktext);
+
 void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 		  const string &str)
 {
@@ -826,14 +834,15 @@ void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 		}
 	}
 	if (iter == match_list.end()) {
-						// Should not reach here
-		throw ErrorGenericFile(_("cannot find directory "
-					 "containing the string %$"),
-				       str);
+						// getcwd fails
+		iter = match_list.begin();	// Arbitrary pick the first
+						// one
+	}
+	else {
+		iter = current;
+		next_loop(match_list, iter);
 	}
 
-	iter = current;
-	next_loop(match_list, iter);
 	while (k_access((*iter).name, F_OK)) {
 		cout << flush;			// Avoid out-of-order display
 		gtout(cerr, _("%$: skipping %$"
@@ -864,7 +873,6 @@ void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 void	CD(const string &linktext, bool entire_tree)
 {
 	LoadDirFile(1);
-
 	HyperDraw *drawObj = 0;
 
 	if (saveCwd.size())				// Current directory is available
@@ -875,21 +883,39 @@ void	CD(const string &linktext, bool entire_tree)
 	
 	GenerateHTMLStrings();
 
-	list<MatchInfo>	match_list;
 	if (linktext.size()) {				// kcd DIR
+
+		ImportBookmarks("");
+		list<MatchInfo>	match_list;
+
+		if (entire_tree)			// Ignore bookmarks
 							// We add current
 							// directory into the mix
 							// to aid cycling through
 							// all match directory
 							// in the command line
-		FindMatchDir(match_list, linktext, true);
+			FindMatchDir(match_list, linktext, true);
 
-		if (!entire_tree
-		    && count_if(match_list.begin(), match_list.end(),
-			        MatchInfoIsBookmarked())) {
-							// Remove entries not
-							// bookmarked
-			match_list.remove_if(MatchInfoIsNotBookmarkedNorCurDir());
+		else {
+			GetBookmarks(match_list, linktext);
+			if (match_list.size()) {
+				if (match_list.size() > 1 && saveCwd.size()) {
+					list<MatchInfo>::iterator iter;
+					for (iter = match_list.begin();
+					     iter != match_list.end(); ++iter) {
+						if ((*iter).name == saveCwd)
+							break;
+					}
+					if (iter == match_list.end()) {
+							// Add current directory to aid jumping
+						FindDirInfo *f = FindDir(saveCwd, true, false);
+						match_list.push_front(MatchInfo(saveCwd, f->dir_ptr, true));
+					}
+				}
+			}
+			else {
+				FindMatchDir(match_list, linktext, true);
+			}
 		}
 
 		int	adjust = 0;			// In case getcwd fails
@@ -1004,50 +1030,66 @@ void	CD(const string &linktext, bool entire_tree)
 	Bookmark helper functions
 *************************************************************************/
 
-void	AddBookmark(const string &str)
+list<string> bookmark_list;
+
+void	AddBookmark(const string &str, bool verify = true)
 {
 	string s = ExpandDirectory(str, false);
-	FindDirInfo *f = FindDir(s, true, false);
-	if (f)
-		f->dir_ptr->bookmarked = true;
-	else {
-		cout << flush;			// Avoid out-of-order display
-		gtout(cerr, _("%$: cannot find %$ in saved directory tree\n"
-			      "        Directory ignored"))
-		     << progName << s;
+	if (verify) {
+		FindDirInfo *f = FindDir(s, true, false);
+		if (f)
+			bookmark_list.push_back(s);
+		else {
+			cout << flush;			// Avoid out-of-order display
+			gtout(cerr, _("%$: cannot find %$ in saved directory tree\n"
+				      "        Directory ignored\n"))
+			     << progName << s;
+		}
+		delete f;
 	}
+	else
+		bookmark_list.push_back(s);
 }
 
-void	RemoveBookmark(const string &str)
+void	RemoveBookmark(const string &str, bool verify = true)
 {
 	string s = ExpandDirectory(str, false);
-	FindDirInfo *f = FindDir(s, true, false);
-	if (f)
-		f->dir_ptr->bookmarked = false;
-	else {
-		cout << flush;			// Avoid out-of-order display
-		gtout(cerr, _("%$: cannot find %$ in saved directory tree\n"
-			      "        Directory ignored"))
-		     << progName << s;
+	if (verify) {
+		FindDirInfo *f = FindDir(s, true, false);
+		if (!f) {
+			cout << flush;			// Avoid out-of-order display
+			gtout(cerr, _("%$: cannot find %$ in saved directory tree\n"
+				      "        Directory ignored\n"))
+			     << progName << s;
+		}
+		delete f;
 	}
+	remove(bookmark_list.begin(), bookmark_list.end(), s);
 }
 
 void	GetBookmarks(list<MatchInfo> &match_list, const string &linktext)
 {
-	LoadDirFile(1);
+	for (list<string>::const_iterator iter = bookmark_list.begin();
+	     iter != bookmark_list.end(); iter++) {
 
-	if (linktext.size()) {
-		FindMatchDir(match_list, linktext);
-							// Remove entries not
-							// bookmarked
-		match_list.remove_if(MatchInfoIsNotBookmarkedNorCurDir());
-	}
-	else {
-		FindBookmarkedDir(match_list);
+		if (linktext.size()) {
+			if (iter->find(linktext) != string::npos) {
+				FindDirInfo *f = FindDir(*iter, true, false);
+				if (f)
+					match_list.push_back(MatchInfo(*iter, f->dir_ptr, false));
+				delete f;
+			}
+		}
+		else {
+			FindDirInfo *f = FindDir(*iter, true, false);
+			if (f)
+				match_list.push_back(MatchInfo(*iter, f->dir_ptr, false));
+			delete f;
+		}
 	}
 }
 
-void	ExportBookmarks(const string &filename, const list<MatchInfo> &match_list)
+void	OutputBookmarks(const string &filename)
 {
 	const char *	s;
 	if (filename.size())
@@ -1060,9 +1102,9 @@ void	ExportBookmarks(const string &filename, const list<MatchInfo> &match_list)
 		throw ErrorGenericFile(_("cannot create file %$"), s);
 	}
 
-	for (list<MatchInfo>::const_iterator iter = match_list.begin();
-	     iter != match_list.end(); iter++) {
-		ofs << iter->name << '\n';
+	for (list<string>::const_iterator iter = bookmark_list.begin();
+	     iter != bookmark_list.end(); iter++) {
+		ofs << *iter << '\n';
 		if (!ofs)
 			throw ErrorGenericFile(_("error writing file %$"), s);
 	}
@@ -1072,10 +1114,9 @@ void	ExportBookmarks(const string &filename, const list<MatchInfo> &match_list)
 	Bookmark functions
 *************************************************************************/
 
-void	ImportBookmarks(const string &filename, bool load)
+void	ImportBookmarks(const string &filename)
 {
-	if (load)
-		LoadDirFile(1);
+	bookmark_list.clear();
 
 	const char *	s;
 	if (filename.size())
@@ -1095,23 +1136,20 @@ void	ImportBookmarks(const string &filename, bool load)
 			break;
 		if (!ifs)
 			throw ErrorGenericFile(_("error reading file %$"), s);
-		AddBookmark(buffer);
+		AddBookmark(buffer, false);
 	}
 }
 
-void	ExportBookmarks(const string &filename, bool load)
+void	ExportBookmarks(const string &filename)
 {
-	if (load)
-		LoadDirFile(1);
-
-	list<MatchInfo>	match_list;
-	GetBookmarks(match_list, "");
-	ExportBookmarks(filename, match_list);
+	ImportBookmarks("");
+	OutputBookmarks(filename);
 }
 
 void	ShowBookmarks(const string &linktext)
 {
 	LoadDirFile(1);
+	ImportBookmarks("");
 
 	list<MatchInfo>	match_list;
 	GetBookmarks(match_list, linktext);
@@ -1195,10 +1233,11 @@ void	ShowBookmarks(const string &linktext)
 void	AddBookmarks(int argc, char *argv[], int argcBegin)
 {
 	LoadDirFile(1);
+	ImportBookmarks("");
 
 	if (argcBegin == argc) {
 		if (saveCwd.size())
-			AddBookmark(saveCwd);
+			AddBookmark(saveCwd, true);
 		else {
 			cout << flush;			// Avoid out-of-order display
 			gtout(cerr, _("%$: cannot determine current directory\n"
@@ -1217,7 +1256,7 @@ void	AddBookmarks(int argc, char *argv[], int argcBegin)
 					string dir = saveCwd;
 					dir += '/';
 					dir += argv[i];
-					AddBookmark(dir);
+					AddBookmark(dir, true);
 				}
 				else {
 					cout << flush;			// Avoid out-of-order display
@@ -1227,21 +1266,21 @@ void	AddBookmarks(int argc, char *argv[], int argcBegin)
 				}
 			}
 			else
-				AddBookmark(argv[i]);
+				AddBookmark(argv[i], true);
 		}
 	}
 
-	WriteDirFile();
-	ExportBookmarks("", false);
+	OutputBookmarks("");
 }
 
 void	RemoveBookmarks(int argc, char *argv[], int argcBegin)
 {
 	LoadDirFile(1);
+	ImportBookmarks("");
 
 	if (argcBegin == argc) {
 		if (saveCwd.size())
-			RemoveBookmark(saveCwd);
+			RemoveBookmark(saveCwd, true);
 		else {
 			cout << flush;			// Avoid out-of-order display
 			gtout(cerr, _("%$: cannot determine current directory\n"
@@ -1260,7 +1299,7 @@ void	RemoveBookmarks(int argc, char *argv[], int argcBegin)
 					string dir = saveCwd;
 					dir += '/';
 					dir += argv[i];
-					RemoveBookmark(dir);
+					RemoveBookmark(dir, true);
 				}
 				else {
 					cout << flush;			// Avoid out-of-order display
@@ -1270,10 +1309,9 @@ void	RemoveBookmarks(int argc, char *argv[], int argcBegin)
 				}
 			}
 			else
-				RemoveBookmark(argv[i]);
+				RemoveBookmark(argv[i], true);
 		}
 	}
 
-	WriteDirFile();
-	ExportBookmarks("", false);
+	OutputBookmarks("");
 }
