@@ -1,6 +1,6 @@
 /*
 	scandir.cc	Scan Directory Functions
-	Copyright (c) 2000,2001 Kriang Lerdsuwanakij
+	Copyright (c) 2000,2001,2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -286,7 +286,6 @@ void	FullScanDir(ScanLog &scan_log)
 	}
 
 	ExpandSymLink(scan_log);
-	ImportBookmarks("", false);
 	WriteDirFile();
 
 	time(&t2);
@@ -652,7 +651,6 @@ void	SmartScanDir(ScanLog &scan_log)
 		dirTree.sort();
 
 	ExpandSymLink(scan_log);
-	ImportBookmarks("", false);
 	WriteDirFile();
 
 	time(&t2);
@@ -691,8 +689,7 @@ void	DoPartialScanDir(ScanLog &scan_log, const string &str_, bool full_scan)
 	}
 
 	string current_dir = dir_info->str;
-	sptr<DirectoryEntry> dir(dir_info->dir_ptr);
-	dir.release();				// We don't own this
+	sptr<DirectoryEntry> dir = dir_info->dir_ptr;	// We don't own this
 
 	scan_log.PrintDirectory(current_dir);
 
@@ -731,7 +728,6 @@ void	PartialScanDir(ScanLog &scan_log, const vector<string> &str_, bool full_sca
 	}
 
 	ExpandSymLink(scan_log);
-	ImportBookmarks("", false);
 	WriteDirFile();
 	
 	time(&t2);
